//----------------------------------------------------------------------------
//
// DETL - A template library 1.0 beta
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Templated array class
//
// Types used in these arrays must have a default constructor and a sensible
// operator=
//
//----------------------------------------------------------------------------
#ifndef TEMPLATE_ARRAY
#define TEMPLATE_ARRAY

//----------------------------------------------------------------------------
#include "detl.h"
#include <assert.h>

//----------------------------------------------------------------------------
// Base class for simple array
template <class T> class Tarray
         {
         protected:
               T* _data;
               int _size;

               // Copy of each individual element
               void clone (const Tarray<T> &other);

         public:
               // Constructor and destructors
               Tarray (void) : _data(NULL),_size(0) { };
               Tarray (const int i) : _data(new T[i]), _size(i) {};
               Tarray (const Tarray<T> &other) : _data(NULL) { clone(other); };
               virtual ~Tarray(void) { delete _data; };

               const Tarray<T>& operator=(const Tarray<T>& other) { clone(other); return *this; };

               // Basic operations
			      inline operator T*() const { return _data; }
			      inline operator T* const () const { return _data; }

               // Allocation manipulation operations
                      void resize (const int r);
               inline void clear  (void) { delete _data; _data = NULL; _size = 0; };

               // Indexing operations
               inline const T& operator [] (const int i) const { assert(i<_size); return _data[i]; };
               inline T& operator [] (const int i) { assert(i<_size); return _data[i]; };

               // Information operations
               inline int size (void) const { return _size; };
         };
//----------------------------------------------------------------------------
// Internal clone operation
template <class T> void Tarray<T>::clone(const Tarray<T> &other)
   {
   // Don't clone over ourselves
   if(&other==this) return;

   // Get some new memory
   delete _data;
   T* _data = new T[other._size];
   assert(_data!=NULL);

   _size = other._size;

   // Copy over the data
   for(int i=0;i<_size;i++) _data[i] = other._data[i];
   }
//----------------------------------------------------------------------------
// Resize the vector
template <class T> void Tarray<T>::resize (const int newsize)
   {
   // Don't change the size if it's the same
   if(newsize==_size) return;

   // Possibly empty everything
   if(newsize==0)
      {
      clear();
      return;
      }
   // Allocate new memory
   T* newdata = new T[newsize];
   assert(newdata!=NULL);

   // Copy stuff over if there was stuff here
   if(_data!=NULL)
      {
      for(int i=0;i<newsize;i++)
         {
         if(i<_size) newdata[i] = _data[i];
         else i = newsize;
         }
      delete _data;
      }
   _size = newsize;
   _data = newdata;
   }
//----------------------------------------------------------------------------
#endif
