//------------------------------------------------------------------------------
//		Log file implementation
//------------------------------------------------------------------------------
#include "logfile.h"
#include <string.h>
#include <assert.h>
#include <errno.h>

//------------------------------------------------------------------------------
// The global logfile
logfile stdlog;

//------------------------------------------------------------------------------
// Send the output somewhere
void logfile::direct (const char *name, int buffer_size)
	{
   // Close anything previously open
   close();

	// Deal with the filename
	if(name==NULL)
		{
		// Send the log to stderr
		strcpy(filename,"stderr");
		stream = stderr;
		}
	else
		{
		// Open a normal file name
		strcpy(filename,name);
		stream = fopen(filename,"wt");
		if(stream==NULL)
         {
         fprintf(stderr,"\nError: %s\n",sys_errlist[errno]);
         stream = stderr;
         }
		}

	// Allocate a buffer if necessary
	if(buffer_size>0)
		{
		// Attach the buffer to the stream
		setvbuf(stream,NULL,_IOFBF,buffer_size);
		}
	}
//..............................................................................
// Destructor
logfile::~logfile (void)
	{
	// Close the stream if it was opened
	if((stream!=NULL)&&(stream!=stderr)) fclose(stream);
	}
//------------------------------------------------------------------------------
// Log an event for character
logfile &logfile::operator << (const char data)
	{
	fprintf(stream,"%c",data);
	return *this;
	}
//..............................................................................
// Log an event for bool
logfile &logfile::operator << (const bool boo)
	{
	if(boo==true) fprintf(stream,"true");
	else fprintf(stream,"false");
	return *this;
	}
//..............................................................................
// Log an event for character string
logfile &logfile::operator << (const char *event)
	{
	fprintf(stream,"%s",event);
	return *this;
	}
//...............................................
// Log an event for float
logfile &logfile::operator << (const float num)
	{
	fprintf(stream,"%f",num);
	return *this;
	}
//...............................................
// Log an event for float
logfile &logfile::operator << (const int num)
	{
	fprintf(stream,"%i",num);
	return *this;
	}
//..............................................
// Log an event for float
logfile &logfile::operator << (const unsigned num)
	{
	fprintf(stream,"%u",num);
	return *this;
	}
//..............................................
// Log an event for long
logfile &logfile::operator << (const long num)
	{
	fprintf(stream,"%li",num);
	return *this;
	}
//..............................................
// Log an event for unsigned long
logfile &logfile::operator << (const unsigned long num)
	{
	fprintf(stream,"%lu",num);
	return *this;
	}
//------------------------------------------------------------------------------
// Close the stream
void logfile::close (void)
	{
	// Close the stream if it's not the stderr stream
	if(stream!=stderr)
		{
		fclose(stream);
		stream = NULL;
		}
	}
//------------------------------------------------------------------------------
// Flush the stream
void logfile::flush (void)
	{
	fflush(stream);
	}
//------------------------------------------------------------------------------
